// This code computes strongly connected components of a graph.
// How to use it:
//   - specify graph using add_edge()
//   - calculate() returns the number of components
//     the group index of vertex v is then stored in group_num[v]

struct scc {
  vector<vector<int> > g, g_rev;
  vector<bool> visited;
  vector<int> group_num;
  int group_cnt;
  stack<int> s;

  scc(size_t n)
    : g(n), g_rev(n), visited(n), group_num(n),
      group_cnt(0) {}
  void add_edge(int a, int b) {
    g[a].push_back(b);
    g_rev[b].push_back(a);
  }
  void fill_forward(int v) {
    visited[v] = true;
    for (int w : g[v])
      if (!visited[w])
	fill_forward(w);
    s.push(v);
  }
  void fill_backward(int v) {
    visited[v] = false;
    group_num[v] = group_cnt;
    for (int w : g_rev[v])
      if (visited[w])
	fill_backward(w);
  }
  int calculate() {
    for (size_t i=0; i<g.size(); ++i)
      if (!visited[i])
	fill_forward(i);
    for (; !s.empty(); s.pop())
      if (visited[s.top()])
	fill_backward(s.top()), ++group_cnt;
    return group_cnt;
  }
};
